/******************** (C) COPYRIGHT 2008 PHYTEC Messtechnik ********************
* File Name          : i2cee.c
* Author             : Marc Dzierzawa
* Version            : V 0.1
* Date               : 17.12.08
* Description        : I2C-EEPROM Functions
********************************************************************************/

#include "stm32f10x_lib.h"
#include "i2cee.h"

/*******************************************************************************
* Function Name  : I2C_EE_Init
* Description    : Init the I2C2 Interface for communication with the EEPROM
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void I2C_EE_Init(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	I2C_InitTypeDef  I2C_InitStructure;

	//GPIO Configuration for I2C2
	//GPIOB Clock Enable
  	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	//I2C2 Clock Enable
  	RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C2, ENABLE);

	
	// Configure I2C2 pins: SCL and SDA 
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_10 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	//Configure I2C2 Interface
	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 0xA0;		//EEPROM-Adress
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 400000;		//400 kHz Clock Speed
 
	//Enable I2C-Interface
	I2C_Cmd(I2C2, ENABLE);
	//Init I2C-Interface
	I2C_Init(I2C2, &I2C_InitStructure);


	   

}


/*******************************************************************************
* Function Name  : I2C_EE_AddrRead
* Description    : Reads data from the specified address.
* Input          : - pBuffer : pointer to the buffer that receives the data read 
*                    from the EEPROM.
*                  - ReadAddr : EEPROM's internal address to read from.
*                  - NumByteToRead : number of bytes to read from the EEPROM.
* Output         : None
* Return         : None
*******************************************************************************/
void I2C_EE_AddrRead(u8* pBuffer, u16 ReadAddr, u16 NumByteToRead)
{ 
  I2C_EE_WRReady();

  //Wait till bus is not busy
  while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY)); 
  
  //Send START condition 
  I2C_GenerateSTART(I2C2, ENABLE);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
   
  // Send EEPROM address for read
  I2C_Send7bitAddress(I2C2, 0xA0, I2C_Direction_Transmitter);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  
  // Send the EEPROM's internal address (MSB) to read from 
  I2C_SendData(I2C2, ((ReadAddr & 0xFF00)>>8));  
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the EEPROM's internal address (LSB) to read from 
  I2C_SendData(I2C2, (ReadAddr & 0x00FF));  
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  
  // Send START condition a second time   
  I2C_GenerateSTART(I2C2, ENABLE);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));
  
  // Send EEPROM address for read 
  I2C_Send7bitAddress(I2C2, 0xA0, I2C_Direction_Receiver);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));
  
  // While there is data to be read 
  while(NumByteToRead--)  
  {
    if(!NumByteToRead)
    {
      // Disable Acknowledgement 
      I2C_AcknowledgeConfig(I2C2, DISABLE);
	   
	  // Send STOP Condition 
  	  I2C_GenerateSTOP(I2C2, ENABLE);
    }     
    while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_RECEIVED));   
	// Read a byte from the EEPROM 
    *pBuffer = I2C_ReceiveData(I2C2);
    pBuffer++;  
  }	

  // Enable Acknowledgement to be ready for another reception 
  I2C_AcknowledgeConfig(I2C2, ENABLE);
}

/*******************************************************************************
* Function Name  : I2C_EE_ByteWrite
* Description    : Writes one byte to the I2C EEPROM.
* Input          : - pBuffer : pointer to the buffer  containing the data to be 
*                    written to the EEPROM.
*                  - WriteAddr : EEPROM's internal address to write to.
* Output         : None
* Return         : None
*******************************************************************************/
void I2C_EE_ByteWrite(u8* pBuffer, u16 WriteAddr)
{
    I2C_EE_WRReady();

  //Wait till bus is not busy
  while(I2C_GetFlagStatus(I2C2, I2C_FLAG_BUSY));  
  // Send START condition 
  I2C_GenerateSTART(I2C2, ENABLE);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_MODE_SELECT));  

  // Send EEPROM address for write
  I2C_Send7bitAddress(I2C2, 0xA0, I2C_Direction_Transmitter);
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
      
  // Send the EEPROM's internal address (MSB) to write to 
  I2C_SendData(I2C2, ((WriteAddr & 0xFF00)>>8));
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the EEPROM's internal address (MSB) to write to 
  I2C_SendData(I2C2, (WriteAddr & 0x00FF));
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));

  // Send the byte to be written 
  I2C_SendData(I2C2, *pBuffer); 
  while(!I2C_CheckEvent(I2C2, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  
  // Send STOP condition 
  I2C_GenerateSTOP(I2C2, ENABLE); 
}

/*******************************************************************************
* Function Name  : I2C_EE_Ready
* Description    : Checks if the EEPROM is ready to communicate 
* Input          : None
* Output         : 0 = Not Ready, other = Ready
* Return         : None
*******************************************************************************/
void I2C_EE_WRReady(void)
{
  vu16 SR1_Tmp = 0;
  do
  {
    /* Send START condition */
    I2C_GenerateSTART(I2C2, ENABLE);
    /* Read I2C1 SR1 register */
    SR1_Tmp = I2C_ReadRegister(I2C2, I2C_Register_SR1);
    /* Send EEPROM address for write */
    I2C_Send7bitAddress(I2C2, 0xA0, I2C_Direction_Transmitter);
  }while(!(I2C_ReadRegister(I2C2, I2C_Register_SR1) & 0x0002));
  
  /* Clear AF flag */
  I2C_ClearFlag(I2C2, I2C_FLAG_AF);
  I2C_GenerateSTOP(I2C2, ENABLE);
}
